// src/redis-cli.tsx
import { useEffect, useRef, useState } from "react";
import * as ScrollArea from "@radix-ui/react-scroll-area";
import { Fragment, jsx, jsxs } from "react/jsx-runtime";
var RedisCli = (props) => {
  const [history, setHistory] = useState([]);
  const [historyIndex, setHistoryIndex] = useState(null);
  const [results, setResults] = useState([]);
  function addCommand(command) {
    command.time ??= Date.now();
    setHistory((prev) => {
      if (prev.length === 0 || prev[0] !== command.command) {
        return [command.command, ...prev];
      }
      return prev;
    });
    setResults((prev) => [...prev, command]);
  }
  const [loading, setLoading] = useState(false);
  const [stdin, setStdin] = useState("");
  const specialCommands = {
    clear: (_ctx) => {
      setResults([]);
      setHistoryIndex(null);
      setStdin("");
    },
    help: (ctx) => {
      ctx.addCommand({
        time: Date.now(),
        command: "help",
        result: /* @__PURE__ */ jsxs("div", { children: [
          /* @__PURE__ */ jsx("p", { children: "You can execute Redis commands in the terminal:" }),
          /* @__PURE__ */ jsx(
            "a",
            {
              className: "upstash-cli-link",
              href: "https://upstash.com/redis-api-compatibility",
              target: "_blank",
              rel: "noreferrer",
              children: "https://upstash.com/redis-api-compatibility"
            }
          ),
          /* @__PURE__ */ jsx("br", {}),
          /* @__PURE__ */ jsxs("div", { className: "upstash-cli-help-command", children: [
            /* @__PURE__ */ jsx("span", { children: "Try" }),
            /* @__PURE__ */ jsx("button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("GET key"), children: "> GET key" }),
            /* @__PURE__ */ jsx("button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("LPUSH list e1 e2"), children: "> LPUSH list" })
          ] }),
          /* @__PURE__ */ jsxs("div", { className: "upstash-cli-help-command", children: [
            /* @__PURE__ */ jsx("span", { children: "Special commands" }),
            /* @__PURE__ */ jsxs("button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("clear"), children: [
              "> clear",
              " "
            ] }),
            /* @__PURE__ */ jsxs("button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("help"), children: [
              "> help",
              " "
            ] })
          ] })
        ] })
      });
    },
    ...props.commands
  };
  async function runRedisCommand(command) {
    try {
      const args = splitArgs(command);
      const res = await fetch(props.url, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
          Authorization: `Bearer ${props.token}`
        },
        body: JSON.stringify(args)
      });
      const json = await res.json();
      addCommand({
        command,
        result: json.error ?? json.result,
        error: Boolean(json.error),
        time: Date.now()
      });
    } catch (error) {
      console.error(error);
      addCommand({ command, error: true, result: error.message, time: Date.now() });
    }
  }
  useEffect(() => {
    if (historyIndex === null) {
      return;
    }
    const cmd = history[historyIndex];
    if (!cmd) {
      return;
    }
    setStdin(cmd);
  }, [historyIndex, history]);
  const ref = useRef(null);
  const onEnter = async () => {
    try {
      if (loading) {
        return;
      }
      setLoading(true);
      const command = stdin.trim();
      if (!command) {
        addCommand({ command, time: Date.now() });
        return;
      }
      if (specialCommands[command]) {
        specialCommands[command]({
          setStdin,
          addCommand
        });
        return;
      }
      await runRedisCommand(command);
    } catch (error) {
      const err = error;
      console.error(err.message);
    } finally {
      setHistoryIndex(null);
      setLoading(false);
      setStdin("");
      setTimeout(() => ref.current?.scrollIntoView({ behavior: "smooth", block: "nearest" }), 10);
    }
  };
  useEffect(() => {
    async function run() {
      if (!props.init) {
        return;
      }
      try {
        const commands = Array.isArray(props.init) ? props.init : [props.init];
        for (const command of commands) {
          const cmd = specialCommands[command];
          if (cmd) {
            cmd({
              setStdin,
              addCommand
            });
          } else {
            await runRedisCommand(command);
          }
        }
      } catch (error) {
        console.error(error);
      }
    }
    run();
  }, [props.init]);
  return /* @__PURE__ */ jsx(
    "div",
    {
      className: `upstash-cli ${props.className}`,
      onMouseUp: () => {
        if (window?.getSelection()?.type !== "Range") {
          ref.current?.focus();
        }
      },
      children: /* @__PURE__ */ jsxs(ScrollArea.Root, { style: { overflow: "hidden" }, children: [
        /* @__PURE__ */ jsxs(ScrollArea.Viewport, { className: "upstash-cli-viewport", children: [
          /* @__PURE__ */ jsx("span", { style: { color: "#00e9a3" }, children: props.welcome ?? /* @__PURE__ */ jsxs(Fragment, { children: [
            "Welcome to Upstash CLI!",
            /* @__PURE__ */ jsx("br", {}),
            "Execute Redis commands online (e.g. 'SET key value'). Type 'help' for more."
          ] }) }),
          results.map((r) => /* @__PURE__ */ jsx(Result, { result: r }, r.time)),
          /* @__PURE__ */ jsx(Line, { className: loading ? "upstash-cli-loading" : "", prefix: /* @__PURE__ */ jsx("span", { children: "\u279C" }), children: /* @__PURE__ */ jsx(
            "input",
            {
              type: "text",
              spellCheck: false,
              style: {
                resize: "none"
              },
              ref,
              value: stdin,
              onChange: (e) => setStdin(e.currentTarget.value),
              onKeyDown: async (e) => {
                if (e.ctrlKey && e.key === "c") {
                  e.preventDefault();
                  addCommand({ command: `${stdin}^C`, time: Date.now() });
                  setStdin("");
                  return;
                }
                if (e.key === "Enter" && !e.shiftKey) {
                  e.preventDefault();
                  await onEnter();
                  return;
                }
                if (e.key === "ArrowUp") {
                  if (history.length === 0) {
                    setHistoryIndex(null);
                    return;
                  }
                  setHistoryIndex(
                    Math.min(history.length - 1, historyIndex === null ? 0 : historyIndex + 1)
                  );
                  return;
                }
                if (e.key === "ArrowDown") {
                  if (history.length === 0) {
                    setHistoryIndex(null);
                    return;
                  }
                  setHistoryIndex(
                    Math.max(0, historyIndex === null ? history.length - 1 : historyIndex - 1)
                  );
                  return;
                }
              },
              className: "upstash-cli-stdin"
            },
            "stdin"
          ) })
        ] }),
        /* @__PURE__ */ jsx(ScrollArea.Scrollbar, { className: "upstash-cli-scrollbar", orientation: "vertical", children: /* @__PURE__ */ jsx(ScrollArea.Thumb, { className: "upstash-cli-scrollbar-thumb" }) }),
        /* @__PURE__ */ jsx(ScrollArea.Corner, { style: { background: "black" } })
      ] })
    }
  );
};
var Line = ({
  className,
  prefix,
  children
}) => /* @__PURE__ */ jsxs("div", { className: ["upstash-cli-line", className].join(" "), children: [
  /* @__PURE__ */ jsx("div", { className: "upstash-cli-line-prefix", children: prefix ?? /* @__PURE__ */ jsx("span", { children: " " }) }),
  /* @__PURE__ */ jsx("div", { className: "upstash-cli-line-content", children })
] });
function splitArgs(input) {
  const separator = /\s/g;
  let singleQuoteOpen = false;
  let doubleQuoteOpen = false;
  let tokenBuffer = [];
  const ret = [];
  const arr = [...input];
  for (const element of arr) {
    const matches = element.match(separator);
    if (element === "'" && !doubleQuoteOpen) {
      singleQuoteOpen = !singleQuoteOpen;
      continue;
    }
    if (element === '"' && !singleQuoteOpen) {
      doubleQuoteOpen = !doubleQuoteOpen;
      continue;
    }
    if (!(singleQuoteOpen || doubleQuoteOpen) && matches) {
      if (tokenBuffer.length > 0) {
        ret.push(tokenBuffer.join(""));
        tokenBuffer = [];
      } else {
        ret.push(element);
      }
    } else {
      tokenBuffer.push(element);
    }
  }
  if (tokenBuffer.length > 0) {
    ret.push(tokenBuffer.join(""));
  } else {
    ret.push("");
  }
  return ret;
}
var Result = ({ result }) => {
  return /* @__PURE__ */ jsxs(Fragment, { children: [
    /* @__PURE__ */ jsx(
      Line,
      {
        prefix: result.error ? /* @__PURE__ */ jsx(
          "span",
          {
            style: {
              color: "#EF4444"
            },
            children: "\u2717"
          }
        ) : /* @__PURE__ */ jsx("span", { children: "\u279C" }),
        children: /* @__PURE__ */ jsx("span", { children: result.command })
      }
    ),
    result.result === void 0 ? null : /* @__PURE__ */ jsx(Line, { children: /* @__PURE__ */ jsx(
      "div",
      {
        className: "upstash-cli-result",
        style: {
          color: result.error ? "#EF4444" : void 0
        },
        children: formatResult(result.result)
      }
    ) })
  ] });
};
function formatResult(result) {
  switch (typeof result) {
    case "undefined": {
      return "";
    }
    case "boolean":
    case "number": {
      return result.toString();
    }
    case "object": {
      if (result === null) {
        return "nil";
      }
      if (Array.isArray(result)) {
        return result.map((e) => formatResult(e)).join(", ");
      }
      return result;
    }
    default: {
      return result;
    }
  }
}
export {
  RedisCli
};
